from openai import OpenAI, AsyncOpenAI
from tqdm import tqdm
import asyncio
from types import SimpleNamespace

class VllmOnlinePredictor:
    def __init__(self, model_path, url="http://localhost:8000/v1", api_key="EMPTY" max_concurrent_requests=256):
        self.client = AsyncOpenAI(api_key=api_key, base_url=url)
        self.model_path = model_path
        self.max_concurrent_requests =  max_concurrent_requests
        print("Warning:sampling_params only use temperature, top_p, max_tokens, n")

    async def _generate_one(self, message, sampling_params, enable_thinking):
        async with self.semaphore:
            try:
                response = await self.client.chat.completions.create(
                    model=self.model_path,
                    messages=message,
                    temperature=sampling_params.temperature,
                    top_p=sampling_params.top_p,
                    max_tokens=sampling_params.max_tokens,
                    n=sampling_params.n,
                    extra_body={
                        "repetition_penalty": sampling_params.repetition_penalty,
                        "chat_template_kwargs": {"enable_thinking": enable_thinking}
                    },
                )
            except Exception as e:
                print(f"Error: {e}")
                response = SimpleNamespace(
                    choices=[SimpleNamespace(message=SimpleNamespace(content="")) for _ in range(sampling_params.n)]
                )
            self.pbar.update(1)
            return response

    async def _generate(self, messages, sampling_params, enable_thinking):
        self.semaphore = asyncio.Semaphore(self.max_concurrent_requests)
        self.pbar = tqdm(total=len(messages), desc="Generating")
        tasks = [
            self._generate_one(message, sampling_params, enable_thinking)
            for message in messages
        ]
        responses = await asyncio.gather(*tasks)
        self.pbar.close()
        return responses

    def generate(self, messages, sampling_params, enable_thinking=False):
        return asyncio.run(self._generate(messages, sampling_params, enable_thinking))



import os
import json
import pandas as pd
import numpy as np

def read_any(path):
    if path.endswith('.json'):
        with open(path, 'r') as f:
            data = json.load(f)
    if path.endswith('.jsonl'):
        with open(path, 'r') as f:
            data = [json.loads(line.strip()) for line in f]
    if path.endswith('.parquet'):
        df = pd.read_parquet(path)
        df = df.map(lambda x: x.tolist() if isinstance(x, np.ndarray) else x)
        data = df.to_dict(orient='records')
    if path.endswith('.csv'):
        df = pd.read_csv(path)
        data = df.to_dict(orient='records')
    return data

def save_jsonl(datas, save_path, exist_ok=True):
    dir_name = os.path.dirname(save_path)
    os.makedirs(dir_name, exist_ok=True)
    if os.path.exists(save_path) and not exist_ok:
        raise("file existed")
    with open(save_path, 'w', encoding="utf-8") as f:
        for data in datas:
            f.write(json.dumps(data, ensure_ascii=False) + "\n")

def save_parquet(datas, save_path, exist_ok=True):
    from datasets import load_dataset, Dataset
    dir_name = os.path.dirname(save_path)
    os.makedirs(dir_name, exist_ok=True)
    if os.path.exists(save_path) and not exist_ok:
        raise("file existed")
    dataset = Dataset.from_list(datas)
    dataset.to_parquet(save_path)
